<?php
/**
 * Add custom login page not accessible from backend
 */
add_action('generate_rewrite_rules', function ($wp_rewrite) {
    $wp_rewrite->rules = array_merge(
        [WOKIT_BACKEND_URL => '/index.php?' . WOKIT_BACKEND_URL . '=1'],
        $wp_rewrite->rules
    );
});
add_filter('query_vars', function ($query_vars) {
    $query_vars[] = WOKIT_BACKEND_URL;
    return $query_vars;
});

function backend_page_template($template)
{
    $backend = intval(get_query_var(WOKIT_BACKEND_URL));
    if ($backend) {
        $new_template = WOKIT_PLUGIN_DIR . '/login-page/login-page.php';
        if ('' != $new_template) {
            add_action('wp_enqueue_scripts', 'register_login_page_styles');
            return $new_template;
        }
    }

    return $template;
}
add_filter('template_include', 'backend_page_template', 99);

function register_login_page_styles()
{
    wp_register_style('login-page', WOKIT_PLUGIN_URL . '/assets/css/login-page/login-page.css');
    wp_enqueue_style('login-page');
}

function redirect_logged_in_user($redirect_to = null)
{
    $user = wp_get_current_user();
    if (user_can($user, 'manage_options')) {
        if ($redirect_to) {
            wp_safe_redirect($redirect_to);
        } else {
            wp_redirect(admin_url());
        }
    } else {
        wp_redirect(home_url(WOKIT_BACKEND_URL));
    }
}

/**
 * Redirect the user to the custom login page instead of wp-login.php.
 */
function redirect_to_custom_login()
{
    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $redirect_to = isset($_REQUEST['redirect_to']) ? $_REQUEST['redirect_to'] : null;

        if (is_user_logged_in()) {
            redirect_logged_in_user($redirect_to);
            exit;
        }

        // The rest are redirected to the login page
        $page = get_bloginfo('url');
        wp_redirect($page);

        //$login_url = home_url( 'backend' );
        //if ( ! empty( $redirect_to ) ) {
        //    $login_url = add_query_arg( 'redirect_to', $redirect_to, $login_url );
        //}

        //wp_redirect( $login_url );
        exit;
    }
}
add_action('login_form_login', 'redirect_to_custom_login');

/**
 * Redirect the user after authentication if there were any errors.
 *
 * @param Wp_User|Wp_Error  $user       The signed in user, or the errors that have occurred during login.
 * @param string            $username   The user name used to log in.
 * @param string            $password   The password used to log in.
 *
 * @return Wp_User|Wp_Error The logged in user, or error information if there were errors.
 */
function maybe_redirect_at_authenticate($user, $username, $password)
{
    // Check if the earlier authenticate filter (most likely,
    // the default WordPress authentication) functions have found errors
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        if (is_wp_error($user)) {
            $error_codes = join(',', $user->get_error_codes());

            $login_url = home_url(WOKIT_BACKEND_URL);
            $login_url = add_query_arg('login', $error_codes, $login_url);

            wp_redirect($login_url);
            exit;
        }
    }

    return $user;
}
add_filter('authenticate', 'maybe_redirect_at_authenticate', 101, 3);

/**
 * Redirect to custom login page after the user has been logged out.
 */
function redirect_after_logout()
{
    $redirect_url = home_url(WOKIT_BACKEND_URL . '?logged_out=true');
    wp_safe_redirect($redirect_url);
    exit;
}
add_action('wp_logout', 'redirect_after_logout');

/**
 * Redirects the user to the custom "Forgot your password?" form
 */
function redirect_to_custom_lostpassword()
{
    if ('GET' == $_SERVER['REQUEST_METHOD']) {
        if (is_user_logged_in()) {
            $this->redirect_logged_in_user();
            exit;
        }

        $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');
        wp_safe_redirect($redirect_url);
        exit;
    }
}
add_action('login_form_lostpassword', 'redirect_to_custom_lostpassword');

/**
 * Initiates password reset.
 */
function do_password_lost()
{
    if ('POST' == $_SERVER['REQUEST_METHOD']) {
        $errors = retrieve_password();
        if (is_wp_error($errors)) {
            // Errors found
            $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');
            $redirect_url = add_query_arg('errors', join(',', $errors->get_error_codes()), $redirect_url);
        } else {
            // Email sent
            $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');
            $redirect_url = add_query_arg('checkemail', 'confirm', $redirect_url);
        }

        wp_redirect($redirect_url);
        exit;
    }
}

add_action('login_form_lostpassword', 'do_password_lost');

/**
 * Resets the user's password if the password reset form was submitted.
 * Redirects to the custom password reset page, or the login page if there are errors.
 */
function do_password_reset()
{
    if ('GET' == $_SERVER['REQUEST_METHOD']) {
        // Verify key / login combo
        $user = check_password_reset_key($_REQUEST['key'], $_REQUEST['login']);
        if (!$user || is_wp_error($user)) {
            if ($user && $user->get_error_code() === 'expired_key') {
                wp_redirect(home_url(WOKIT_BACKEND_URL . '?lost_pass=true&login=expiredkey'));
            } else {
                wp_redirect(home_url(WOKIT_BACKEND_URL . '?lost_pass=true&login=invalidkey'));
            }
            exit;
        }

        $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');
        $redirect_url = add_query_arg('login', esc_attr($_REQUEST['login']), $redirect_url);
        $redirect_url = add_query_arg('key', esc_attr($_REQUEST['key']), $redirect_url);

        wp_redirect($redirect_url);
        exit;
    }

    if ('POST' == $_SERVER['REQUEST_METHOD']) {
        $rp_key = $_REQUEST['rp_key'];
        $rp_login = $_REQUEST['rp_login'];

        $user = check_password_reset_key($rp_key, $rp_login);

        if (!$user || is_wp_error($user)) {
            if ($user && $user->get_error_code() === 'expired_key') {
                wp_redirect(home_url(WOKIT_BACKEND_URL . '?lost_pass=true&login=expiredkey'));
            } else {
                wp_redirect(home_url(WOKIT_BACKEND_URL . '?lost_pass=true&login=invalidkey'));
            }
            exit;
        }

        if (isset($_POST['pass1'])) {
            if ($_POST['pass1'] != $_POST['pass2']) {
                // Passwords don't match
                $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');

                $redirect_url = add_query_arg('key', $rp_key, $redirect_url);
                $redirect_url = add_query_arg('login', $rp_login, $redirect_url);
                $redirect_url = add_query_arg('error', 'password_reset_mismatch', $redirect_url);

                wp_redirect($redirect_url);
                exit;
            }

            if (empty($_POST['pass1'])) {
                // Password is empty
                $redirect_url = home_url(WOKIT_BACKEND_URL . '?lost_pass=true');

                $redirect_url = add_query_arg('key', $rp_key, $redirect_url);
                $redirect_url = add_query_arg('login', $rp_login, $redirect_url);
                $redirect_url = add_query_arg('error', 'password_reset_empty', $redirect_url);

                wp_redirect($redirect_url);
                exit;
            }

            // Parameter checks OK, reset password
            reset_password($user, $_POST['pass1']);
            wp_redirect(home_url(WOKIT_BACKEND_URL . '?password=changed'));
        } else {
            echo "Invalid request.";
        }

        exit;
    }
}
add_action('login_form_rp', 'do_password_reset');
add_action('login_form_resetpass', 'do_password_reset');