<?php

/**
 * Plugin Name: Elementor Custom Widgets
 * Description: Custom Widgets für Elementor
 * Plugin URI:  https://www.zwei14.de
 * Version:     1.1.0
 * Author:      zwei14 GmbH
 * Author URI:  https://www.zwei14.de
 * Text Domain: elementor-custom-widgets
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}

define('ECW_VERSION', '1.1.0');
//define('ECW_PLUGIN_DIR', plugin_dir_path(__FILE__));
//define('ECW_PLUGIN_URL', plugin_dir_url(__FILE__));
define('MINIMUM_ELEMENTOR_VERSION', '3.0.0');
define('MINIMUM_PHP_VERSION', '7.0');


/**
 * Main Elementor Extension Class
 *
 * The main class that initiates and runs the plugin.
 *
 * @since 1.0.0
 */
final class Elementor_Custom_Widgets
{
	/**
	 * Instance
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @static
	 *
	 * @var Elementor_Custom_Widgets The single instance of the class.
	 */
	private static $_instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @static
	 *
	 * @return Elementor_Custom_Widgets An instance of the class.
	 */
	public static function instance()
	{
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function __construct()
	{
		add_action('init', [$this, 'i18n']);
		add_action('plugins_loaded', [$this, 'init']);
		add_action('elementor/elements/categories_registered', [$this, 'add_elementor_widget_categories']);
	}

	/**
	 * Load Textdomain
	 *
	 * Load plugin localization files.
	 *
	 * Fired by `init` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function i18n()
	{
		load_plugin_textdomain('wokit-custom');
	}

	/**
	 * Check if the current user can edit options if so, add the settings page to the elementor main backend menu
	 * 
	 */
	public function elementor_custom_settings()
	{
		if (!current_user_can( 'manage_options' )) {
			return;
		}

		require_once WOKIT_PLUGIN_DIR . '/settings-page/elementor-custom-settings.php';
	}

	/**
	 * Initialize the plugin
	 *
	 * Load the plugin only after Elementor (and other plugins) are loaded.
	 * Checks for basic plugin requirements, if one check fail don't continue,
	 * if all check have passed load the files required to run the plugin.
	 *
	 * Fired by `plugins_loaded` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init()
	{
		// Check if Elementor installed and activated
		if (!did_action('elementor/loaded')) {
			add_action('admin_notices', [$this, 'admin_notice_missing_main_plugin']);
			return;
		}

		// Check for required Elementor version
		if (!version_compare(ELEMENTOR_VERSION, MINIMUM_ELEMENTOR_VERSION, '>=')) {
			add_action('admin_notices', [$this, 'admin_notice_minimum_elementor_version']);
			return;
		}

		// Check for required PHP version
		if (version_compare(PHP_VERSION, MINIMUM_PHP_VERSION, '<')) {
			add_action('admin_notices', [$this, 'admin_notice_minimum_php_version']);
			return;
		}

		// Add Plugin actions and load custom settings page to elementor
		add_action('init', [$this, 'elementor_custom_settings']);
		add_action('elementor/widgets/widgets_registered', [$this, 'init_widgets']);

		// Register widget styles and scripts globally if needed
		//add_action( 'elementor/frontend/after_register_scripts', [ $this, 'global_widget_styles' ] );
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have Elementor installed or activated.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_missing_main_plugin()
	{
		if (isset($_GET['activate'])) unset($_GET['activate']);

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__('"%1$s" benötigt das Plugin: "%2$s".', 'elementor-custom-widgets'),
			'<strong>' . esc_html__('Elementor Custom Widgets', 'elementor-custom-widgets') . '</strong>',
			'<strong>' . esc_html__('Elementor', 'elementor-custom-widgets') . '</strong>'
		);

		printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required Elementor version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_elementor_version()
	{
		if (isset($_GET['activate'])) unset($_GET['activate']);

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
			esc_html__('"%1$s" benötigt "%2$s" in der Version %3$s oder größer.', 'elementor-custom-widgets'),
			'<strong>' . esc_html__('Elementor Custom Widgets', 'elementor-custom-widgets') . '</strong>',
			'<strong>' . esc_html__('Elementor', 'elementor-custom-widgets') . '</strong>',
			MINIMUM_ELEMENTOR_VERSION
		);

		printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required PHP version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_php_version()
	{
		if (isset($_GET['activate'])) unset($_GET['activate']);

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__('"%1$s" benötigt "%2$s" Version %3$s oder größer.', 'elementor-custom-widgets'),
			'<strong>' . esc_html__('Elementor Custom Widgets', 'elementor-custom-widgets') . '</strong>',
			'<strong>' . esc_html__('PHP', 'elementor-custom-widgets') . '</strong>',
			MINIMUM_PHP_VERSION
		);

		printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
	}

	/**
	 * Widget Category
	 *
	 * Add additional widget category.
	 *
	 * @since 1.1.0
	 * @access public
	 */
	public function add_elementor_widget_categories($elements_manager)
	{
		//aqb has no special meaning, just starting with an "a" to have the category on top
		$category_prefix = 'aqb-';

		$elements_manager->add_category(
			$category_prefix . 'widgets',
			[
				'title' => __('zwei14 Elemente', 'elementor-custom-widgets'),
				'icon' => 'fa fa-plug',
			]
		);

		$reorder_cats = function () use ($category_prefix) {
			uksort($this->categories, function ($keyOne, $keyTwo) use ($category_prefix) {
				if (substr($keyOne, 0, 4) == $category_prefix) {
					return -1;
				}
				if (substr($keyTwo, 0, 4) == $category_prefix) {
					return 1;
				}
				return 0;
			});
		};
		$reorder_cats->call($elements_manager);
	}


	/**
	 * Global widget styles
	 *
	 * Load required plugin style files globally.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function global_widget_styles()
	{
		wp_enqueue_style('elementor-styles', plugins_url(WOKIT_PLUGIN_DIR . '/assets/css/xxx.css'));
	}


	/**
	 * Load all Widgets from the /widgets/* folder
	 * 
	 * Non .php and files with an underscore at the beginning are excluded
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 */
	public function load_elementor_widgets()
	{
		$widgets = WOKIT_PLUGIN_DIR . '/elementor-widgets';
		$widgets = array_values(preg_grep('/^((?!\/_).)*.php$/', glob($widgets . '/*.*')));
		$classArray = [];

		foreach ($widgets as $key => $widget) {
			require_once($widget);
			$class = get_declared_classes();
			$classArray[] = end($class);
		}
		return $classArray;
	}


	/**
	 * Init Widgets
	 *
	 * Include widgets files and register them
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init_widgets()
	{
		// Include Widget files		
		foreach ($this->load_elementor_widgets() as $currentClass) {
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new $currentClass());
		}
	}
}

Elementor_Custom_Widgets::instance();
